import GPLinearODEMaker.powers_of_negative_one

"""
    rq_per_kernel(hyperparameters, δ, dorder; shift_ind=0)

Created by kernel_coder(). Requires 3 hyperparameters.
Likely created using rq_per_kernel_base() as an input.
Use with include("src/kernels/rq_per_kernel.jl").

# Arguments
- `hyperparameters::Vector`: The hyperparameter values. For this kernel, they should be `["α", "λ", "rq_per_P"]`
- `δ::Real`: The difference between the inputs (e.g. `t1 - t2`)
- `dorder::Vector{<:Integer}`: How many times to differentiate with respect to the inputs and the `hyperparameters` (e.g. `dorder=[0, 1, 0, 2]` would correspond to differentiating once w.r.t the second input and twice w.r.t `hyperparameters[2]`)
- `shift_ind::Integer=0`: If changed, the index of which hyperparameter is the `δ` shifting one
"""
function rq_per_kernel(
    hyperparameters::Vector{<:Real},
    δ::Real,
    dorder::Vector{<:Integer};
    shift_ind::Integer=0)

    include_shift = shift_ind!=0
    @assert length(hyperparameters)==3+Int(include_shift) "hyperparameters is the wrong length"
    dorder_len = 5 + Int(include_shift)
    @assert length(dorder)==dorder_len "dorder is the wrong length"
    @assert maximum(dorder) < 3 "No more than two derivatives for each time or hyperparameter can be calculated"

    dorder2 = dorder[2]
    dorder[2] += dorder[1]

    if include_shift
        dorder[2] += dorder[shift_ind+2]
        δ += hyperparameters[shift_ind]
        hyperparameters_view = view(hyperparameters, 1:4 .!= shift_ind)
        dorder_view = view(dorder, 2:dorder_len)
        dorder_view = view(dorder_view, 1:(dorder_len-1) .!= (shift_ind+1))
    else
        hyperparameters_view = hyperparameters
        dorder_view = view(dorder, 2:dorder_len)
    end

    α = hyperparameters_view[1]
    λ = hyperparameters_view[2]
    rq_per_P = hyperparameters_view[3]

    if dorder_view==[6, 0, 0, 2]
        func = 2688*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^8 - 2688*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^8 - 256*δ^2*λ*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^10 + 256*δ^2*λ*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^10 + 40320*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^8) + 40320*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^8) + 3584*δ*λ*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^9 + 40320*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 40320*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 252672*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^8) - 16128*δ^2*λ^2*(-1 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^10) - 16128*δ^2*λ^2*(-1 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^10) + 927360*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^8) - 927360*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^8) - 225792*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^9) + 225792*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^9) - 53760*δ^2*λ^3*(-1 - α)*(-2 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^10) + 53760*δ^2*λ^3*(-1 - α)*(-2 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^10) + 97792*δ^2*λ^2*(-1 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^10) - 483840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^8) + 1827840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^8) - 483840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^8) + 752640*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^9) - 2809856*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^9) + 752640*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^9) - 26880*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^10) - 26880*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*cos(δ*π/rq_per_P)^8/(α^3*rq_per_P^10) - 935424*δ^2*λ^3*(-1 - α)*(-2 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^10) + 935424*δ^2*λ^3*(-1 - α)*(-2 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^10) - 645120*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^8) + 645120*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^8) - 376320*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^9) + 4641280*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^9) - 4641280*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^9) + 376320*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^9) + 1397760*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^10) - 4118016*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^10) + 1397760*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^10) - 172032*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^8) - 1505280*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^9) + 5017600*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^9) - 1505280*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^9) - 430080*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^8/(α^4*rq_per_P^10) + 3870720*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^10) - 3870720*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^10) + 430080*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^10) - 1204224*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^9) + 1204224*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^9) - 860160*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^8/(α^5*rq_per_P^10) + 2637824*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^10) - 860160*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^10) - 229376*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^9) - 458752*δ^2*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^8/(α^6*rq_per_P^10) + 458752*δ^2*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^10) - 65536*δ^2*λ^8*(-6 - α)*(-7 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^8*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-8 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^8/(α^7*rq_per_P^10)
    end

    if dorder_view==[5, 0, 0, 2]
        func = 768*δ*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^8 - 768*δ*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^8 - 1920*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^7 + 256*δ^2*λ*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^9 + 28800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^7) - 28800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^7) + 11520*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^8) + 11520*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^8) - 28800*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^7) + 134400*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^7) - 28800*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^7) + 11520*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 11520*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 72192*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^8) - 16128*δ^2*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^9) + 16128*δ^2*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^9) - 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^7) + 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^7) + 264960*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^8) - 264960*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^8) + 53760*δ^2*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^9) - 200704*δ^2*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^9) + 53760*δ^2*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^9) - 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^7) - 138240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^8) + 522240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^8) - 138240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^8) - 26880*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^9) + 331520*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^9) - 331520*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^9) + 26880*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^9) - 184320*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^8) + 184320*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^8) - 107520*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^9) + 358400*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^9) - 107520*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^9) - 49152*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^8) - 86016*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^9) + 86016*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^9) - 16384*δ^2*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^9)
    end

    if dorder_view==[4, 0, 0, 2]
        func = -320*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 + 320*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 64*δ^2*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^8 - 64*δ^2*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^8 - 960*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 960*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 640*δ*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^7 + 7040*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 960*δ^2*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^8) + 960*δ^2*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^8) - 7680*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 7680*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 9600*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^7) - 9600*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^7) + 960*δ^2*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 960*δ^2*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^8) - 6016*δ^2*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^8) - 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 9600*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^7) + 44800*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^7) - 9600*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^7) + 22080*δ^2*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^8) - 22080*δ^2*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^8) - 25600*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^7) + 25600*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^7) - 11520*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^8) + 43520*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^8) - 11520*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^8) - 10240*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^7) - 15360*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^8) + 15360*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^8) - 4096*δ^2*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^8)
    end

    if dorder_view==[3, 0, 0, 2]
        func = -128*δ*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 + 128*δ*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 192*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 64*δ^2*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^7 - 576*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 576*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 384*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 384*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 768*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 2816*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 960*δ^2*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^7) - 960*δ^2*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^7) - 3072*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 3072*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 960*δ^2*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^7) + 4480*δ^2*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^7) - 960*δ^2*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^7) - 2048*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 2560*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^7) + 2560*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^7) - 1024*δ^2*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^7)
    end

    if dorder_view==[2, 0, 0, 2]
        func = 24*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 - 24*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 16*δ^2*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 + 16*δ^2*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 96*δ*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 96*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 48*δ^2*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 48*δ^2*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 288*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 288*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) + 352*δ^2*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 384*δ*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 384*δ^2*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 384*δ^2*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 256*δ^2*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6)
    end

    if dorder_view==[1, 0, 0, 2]
        func = 16*δ*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 - 16*δ*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 8*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 + 16*δ^2*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 64*δ*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 48*δ^2*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 48*δ^2*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 64*δ^2*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5)
    end

    if dorder_view==[0, 0, 0, 2]
        func = 4*δ^2*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 - 4*δ^2*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 8*δ*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 - 16*δ^2*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4)
    end

    if dorder_view==[6, 0, 1, 1]
        func = -384*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 384*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 - 256*δ*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^8 - 12288*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 11520*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 28800*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 17280*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 72960*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) - 11520*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^7) - 408960*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 469632*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 32256*δ*λ*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^8) - 32768*δ*λ*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^8) + 288000*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 1309440*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 541440*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 161280*δ*λ^2*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) + 634368*δ*λ^2*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) - 193536*δ*λ^2*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) + 599040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 983040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 138240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^7) + 107520*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) - 1433600*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) + 1727488*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) - 215040*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) + 331776*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 184320*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^7) + 591360*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) - 2455040*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) + 1200640*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 53760*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)/(α^4*rq_per_P^8) + 49152*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^7) + 731136*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) - 1232896*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) + 215040*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)^3/(α^5*rq_per_P^8) + 286720*δ*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) - 172032*δ*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)^5/(α^6*rq_per_P^8) + 32768*δ*λ^7*(-6 - α)*(-7 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-8 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)^7/(α^7*rq_per_P^8)
    end

    if dorder_view==[5, 0, 1, 1]
        func = -64*δ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 64*δ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 + 320*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 - 9600*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 10240*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) - 2048*δ*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 1920*δ*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) + 14400*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 76800*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 24000*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 4800*δ*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 2880*δ*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 12160*δ*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) + 60800*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 96000*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 9600*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^6) - 1920*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^7) - 68160*δ*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 78272*δ*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 51200*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 25600*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^6) + 48000*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 218240*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 90240*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) + 10240*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^6) + 99840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 163840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 23040*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^7) + 55296*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 30720*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^7) + 8192*δ*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^7)
    end

    if dorder_view==[4, 0, 1, 1]
        func = 64*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 64*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 + 64*δ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 + 512*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 384*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 384*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^5) - 2944*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 4992*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 7424*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 1920*δ*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 2048*δ*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) + 7168*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 3072*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^5) + 2880*δ*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 15360*δ*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 4800*δ*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) + 2048*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^5) + 12160*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 19200*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 1920*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^6) + 10240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 5120*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^6) + 2048*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^6)
    end

    if dorder_view==[3, 0, 1, 1]
        func = 16*δ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 16*δ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 - 48*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 + 288*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 384*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 128*δ*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 96*δ*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 864*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 288*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^4) + 96*δ*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^5) - 736*δ*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 384*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^4) + 1248*δ*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 1856*δ*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) + 1792*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 768*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^5) + 512*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^5)
    end

    if dorder_view==[2, 0, 1, 1]
        func = -8*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 8*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 - 16*δ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 - 16*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^3) + 80*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3) + 64*λ^2*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^3) + 96*δ*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 128*δ*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 288*δ*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 96*δ*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^4) + 128*δ*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^4)
    end

    if dorder_view==[1, 0, 1, 1]
        func = -4*δ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 4*δ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 + 4*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2 + 8*λ*(-1 - α)*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^2) - 8*δ*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^3) + 40*δ*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3) + 32*δ*λ^2*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^3)
    end

    if dorder_view==[0, 0, 1, 1]
        func = 4*δ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2 + 8*δ*λ*(-1 - α)*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^2)
    end

    if dorder_view==[6, 1, 0, 1]
        func = 5760*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) + 5760*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 384*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 384*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 + 64*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 5760*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 5760*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 5760*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 5760*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 5760*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 5760*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 36096*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) + 11520*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 11520*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 36096*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 132480*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) - 132480*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 132480*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) - 132480*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^6) - 5760*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 5760*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 15872*δ*λ^2*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^8) + 15872*δ*λ^2*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^8) - 256*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^8 + 264960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 264960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 69120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 261120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 69120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 69120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 261120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 69120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 69120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 261120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 69120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) + 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^6) - 5760*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 5760*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 6016*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^6) + 36096*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) - 15872*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) + 15872*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) + 53760*δ*λ^3*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) - 200704*δ*λ^3*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) + 53760*δ*λ^3*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) + 49920*δ*λ^3*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) - 176640*δ*λ^3*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) + 49920*δ*λ^3*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) - 207360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 783360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 207360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^7) - 92160*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 92160*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 92160*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 92160*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 92160*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 92160*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^6) - 132480*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 132480*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 103680*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) - 377344*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) + 103680*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) - 26880*δ*λ^4*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) + 331520*δ*λ^4*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) - 331520*δ*λ^4*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) + 26880*δ*λ^4*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) - 26880*δ*λ^4*(-1 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) + 331520*δ*λ^4*(-1 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) - 331520*δ*λ^4*(-1 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) + 26880*δ*λ^4*(-1 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) - 23040*δ*λ^4*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) + 243200*δ*λ^4*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) - 243200*δ*λ^4*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) + 23040*δ*λ^4*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) + 15872*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^8) - 15872*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^8) + 256*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^8) - 256*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^8) - 368640*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) + 368640*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^7) - 24576*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 24576*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 24576*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 24576*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 24576*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 261120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 76800*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) + 906240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) - 906240*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) + 76800*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) + 358400*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) + 358400*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) + 358400*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) - 107520*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 61440*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) + 184320*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) - 61440*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 49920*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) + 176640*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) - 49920*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) - 3840*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) + 24064*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) - 3840*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) - 122880*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^7) + 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-5 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-5 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-5 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-5 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 384000*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 1259520*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) - 384000*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^5*rq_per_P^8) - 86016*δ*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 86016*δ*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) - 86016*δ*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 86016*δ*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) - 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) - 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) - 24576*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) + 24576*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) + 23040*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) - 243200*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) + 243200*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) - 23040*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) + 3840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) - 88320*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) + 88320*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) - 3840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-6 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-6 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^5*rq_per_P^6) + 24576*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 368640*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) + 368640*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^6*rq_per_P^8) - 16384*δ*λ^7*(-6 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) - 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) - 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) - 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) - 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8) + 61440*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) - 184320*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) + 61440*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) + 46080*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) - 174080*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) + 46080*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) - 81920*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^7*rq_per_P^8) + 24576*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) - 24576*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) + 61440*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) - 61440*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) + 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8)
    end

    if dorder_view==[5, 1, 0, 1]
        func = 4800*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) - 4800*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) + 960*δ*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) + 960*δ*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 64*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 64*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 + 320*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 - 64*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 + 4800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 4800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 4800*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) + 22400*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 4800*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 4800*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) + 22400*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 4800*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) + 960*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 960*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 960*δ*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 960*δ*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 960*δ*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 960*δ*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) - 5760*δ*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) - 9600*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 44800*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 9600*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^6) - 12800*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 12800*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 12800*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 12800*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 12800*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 12800*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^5) - 4800*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 4800*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) + 1920*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 1920*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 5760*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 22080*δ*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) - 22080*δ*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 18240*δ*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) - 18240*δ*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) - 960*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 960*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 38400*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) + 38400*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^6) - 5120*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 5120*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 5120*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 5120*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^5) + 4800*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 22400*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 4800*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) + 40320*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 40320*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 11520*δ*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 43520*δ*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 11520*δ*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 11520*δ*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 43520*δ*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 11520*δ*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 7680*δ*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) + 25600*δ*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) - 7680*δ*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 960*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 960*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 5760*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) + 256*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) - 20480*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^6) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 12800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 12800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 30720*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 112640*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 30720*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^7) - 15360*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 15360*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 15360*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 15360*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 15360*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 15360*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 5120*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) + 5120*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) - 18240*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 18240*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) - 3840*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 3840*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 5120*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-5 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-5 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 51200*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) + 51200*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^7) - 4096*δ*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 4096*δ*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 4096*δ*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) - 4096*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) + 7680*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 25600*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 7680*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) + 3840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 17920*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 3840*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 16384*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^7) + 5120*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 5120*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 10240*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 10240*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 4096*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7)
    end

    if dorder_view==[4, 1, 0, 1]
        func = -192*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) - 192*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 - 16*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 192*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 192*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 1408*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 1408*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 1536*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 1536*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 1536*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 1536*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^4) + 192*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 192*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 896*δ*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) - 896*δ*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) + 64*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 - 3072*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) + 3072*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^5) - 1024*λ^4*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 1024*λ^4*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 1024*λ^4*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) + 352*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^4) - 1408*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 896*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 896*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 960*δ*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) + 4480*δ*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 960*δ*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 768*δ*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) + 3072*δ*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) - 768*δ*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 3072*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^5) - 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^4) + 1536*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 1536*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 1728*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 7552*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 1728*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^6) - 2560*δ*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 2560*δ*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 2560*δ*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 2560*δ*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 1024*δ*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) + 1024*δ*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 896*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 896*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) - 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-4 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-4 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 1024*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 6144*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) + 6144*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^6) - 1024*δ*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 1024*δ*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) - 1024*δ*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) + 768*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 3072*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 768*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) + 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 1408*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 3072*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^6) + 1024*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 1024*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 1536*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 1536*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 1024*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6)
    end

    if dorder_view==[3, 1, 0, 1]
        func = -144*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) + 144*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) - 48*δ*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) - 48*δ*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 - 48*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 + 16*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 - 144*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) + 144*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^4) - 192*λ^3*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 192*λ^3*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 48*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 48*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 288*δ*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) - 384*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^4) - 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^3) + 144*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 144*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 288*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 384*δ*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 384*δ*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 192*δ*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) + 192*δ*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) + 48*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 48*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) - 64*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-3 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-3 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α^2*rq_per_P^3) + 192*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 576*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) + 576*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^5) - 256*δ*λ^4*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 256*δ*λ^4*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) - 288*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) - 64*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) - 512*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^5) + 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) + 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 192*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) + 256*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5)
    end

    if dorder_view==[2, 1, 0, 1]
        func = -8*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 8*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 + 4*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 - 32*λ^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3) - 32*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^3) - 32*δ*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) + 32*δ*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) - 16*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 - 16*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-2 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-2 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))/(α*rq_per_P^2) + 32*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3) - 32*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) + 32*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^4) - 64*δ*λ^3*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 64*δ*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^4) + 32*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 32*δ*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 64*δ*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4)
    end

    if dorder_view==[1, 1, 0, 1]
        func = -4*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 4*δ*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 + 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2 - 4*λ*(4*δ*λ*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2) - 8*δ*λ^2*(-1 - α)*π*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2*rq_per_P^2) + 4*δ*λ*(-1 - α)*π*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)*rq_per_P^2))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P + 16*δ*λ^2*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3)
    end

    if dorder_view==[0, 1, 0, 1]
        func = 8*δ*λ^2*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^2) + 4*δ*λ*(2*λ*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2
    end

    if dorder_view==[6, 0, 0, 1]
        func = -384*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 384*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 - 5760*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 5760*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 256*δ*λ*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^8 - 5760*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 5760*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 36096*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) - 132480*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 132480*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 16128*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^8) - 16128*δ*λ^2*(-1 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^8) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 261120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) - 53760*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^8) + 200704*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^8) - 53760*δ*λ^3*(-1 - α)*(-2 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^8) + 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 92160*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 26880*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^7/(α^3*rq_per_P^8) - 331520*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^8) + 331520*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^8) - 26880*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^8) + 24576*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7) + 107520*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^7/(α^4*rq_per_P^8) - 358400*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^8) + 107520*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^8) + 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^7/(α^5*rq_per_P^8) - 86016*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^8) + 16384*δ*λ^7*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^7*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^7/(α^6*rq_per_P^8)
    end

    if dorder_view==[5, 0, 0, 1]
        func = -64*δ*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^7 + 64*δ*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^7 + 320*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 - 4800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 4800*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) - 960*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^7) - 960*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^7) + 4800*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 22400*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 4800*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) - 960*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 960*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^7) + 6016*δ*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^7) + 12800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 12800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) - 22080*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^7) + 22080*δ*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^7) + 5120*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6) + 11520*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^7) - 43520*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^7) + 11520*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^7) + 15360*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^7) - 15360*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^7) + 4096*δ*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^7)
    end

    if dorder_view==[4, 0, 0, 1]
        func = 64*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 64*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 + 192*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 192*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 64*δ*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^6 - 1408*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 1536*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 1536*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) - 960*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^6) + 960*δ*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^6) + 1024*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5) + 960*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^6) - 4480*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^6) + 960*δ*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^6) + 2560*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^6) - 2560*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^6) + 1024*δ*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^6)
    end

    if dorder_view==[3, 0, 0, 1]
        func = 16*δ*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^5 - 16*δ*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^5 - 48*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 + 144*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 144*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 48*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 48*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^5) + 192*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4) - 352*δ*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^5) + 384*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^5) - 384*δ*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^5) + 256*δ*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^5)
    end

    if dorder_view==[2, 0, 0, 1]
        func = -8*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 8*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 - 16*δ*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^4 + 32*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3) + 48*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^4) - 48*δ*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^4) + 64*δ*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^4)
    end

    if dorder_view==[1, 0, 0, 1]
        func = -4*δ*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^3 + 4*δ*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^3 + 4*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2 + 16*δ*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^3)
    end

    if dorder_view==[0, 0, 0, 1]
        func = 4*δ*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^2
    end

    if dorder_view==[6, 0, 2, 0]
        func = 2176*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 1920*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 12288*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 13696*λ*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 5760*λ*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 15360*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^6) + 140160*λ*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 180864*λ*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 3840*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^10/(α^4*rq_per_P^6) - 149760*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 791040*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 427264*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 495360*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 1091840*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 272640*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 476160*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 481280*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 46080*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^10*cos(δ*π/rq_per_P)^2/(α^5*rq_per_P^6) - 159744*λ^5*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 61440*λ^5*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^10*cos(δ*π/rq_per_P)^4/(α^6*rq_per_P^6) - 16384*λ^6*(-6 - α)*(-7 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-8 - α)*sin(δ*π/rq_per_P)^10*cos(δ*π/rq_per_P)^6/(α^7*rq_per_P^6)
    end

    if dorder_view==[5, 0, 2, 0]
        func = 1920*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 2176*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 5760*λ*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 34560*λ*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 13696*λ*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 42240*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 88320*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 15360*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 65280*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 58880*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 3840*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)/(α^4*rq_per_P^5) - 30720*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 10240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)^3/(α^5*rq_per_P^5) - 4096*λ^5*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^9*cos(δ*π/rq_per_P)^5/(α^6*rq_per_P^5)
    end

    if dorder_view==[4, 0, 2, 0]
        func = -160*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 96*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 768*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 448*λ*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^4) - 192*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^4) - 2688*λ*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 5184*λ*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 7872*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 6016*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 5632*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) + 1536*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^4) - 1024*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^4)
    end

    if dorder_view==[3, 0, 2, 0]
        func = -96*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 160*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 768*λ*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 448*λ*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) - 960*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) + 192*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^3) - 256*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^3)
    end

    if dorder_view==[2, 0, 2, 0]
        func = 16*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^2) - 48*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) + 16*λ*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^2) - 144*λ*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) - 64*λ^2*(-1 - α)*(-2 - α)*(-3 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^2)
    end

    if dorder_view==[1, 0, 2, 0]
        func = -16*(-1 - α)*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P) - 16*λ*(-1 - α)*(-2 - α)*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P)
    end

    if dorder_view==[0, 0, 2, 0]
        func = -4*(-1 - α)*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/α
    end

    if dorder_view==[6, 1, 1, 0]
        func = 64*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 - 1920*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 1920*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^2/rq_per_P^6 - 1920*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 1920*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 4800*λ^2*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 2880*λ^2*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 2880*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 2880*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 12032*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 7680*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 5760*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 1920*λ^3*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^6) - 1920*λ^3*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^6) + 12032*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 68160*λ^2*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 78272*λ^2*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 66240*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 66240*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 1920*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 1920*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 128*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 3840*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^4*rq_per_P^6) - 134400*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 144512*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 48000*λ^3*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 218240*λ^3*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 90240*λ^3*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 48000*λ^3*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 218240*λ^3*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 90240*λ^3*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 46080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 174080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 46080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 2880*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 2880*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 1920*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 12032*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 128*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 6016*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 142080*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 610560*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 226560*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 99840*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 163840*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 23040*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 99840*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 163840*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 99840*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 163840*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^6) + 66240*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 66240*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 1920*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 12032*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 376320*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 568320*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^5*rq_per_P^6) + 55296*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 30720*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 55296*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 55296*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 55296*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 24576*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 46080*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 174080*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 46080*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 44160*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 44160*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 245760*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 122880*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^6*rq_per_P^6) + 8192*λ^6*(-6 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 8192*λ^6*(-6 - α)*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 8192*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 8192*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 8192*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 76800*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 23040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 87040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 23040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 40960*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^7*rq_per_P^6) - 24576*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-6 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6)
    end

    if dorder_view==[5, 1, 1, 0]
        func = -64*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 1920*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 1920*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 64*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 1920*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 1920*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 2880*λ^2*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 15360*λ^2*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 4800*λ^2*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 2880*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 13440*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 2880*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 5760*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 28800*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 7680*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) + 12160*λ^3*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 19200*λ^3*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 1920*λ^3*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) + 12160*λ^3*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 19200*λ^3*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 1920*λ^3*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) + 10240*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 10240*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 1920*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 1920*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 128*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) + 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) + 34560*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 48640*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 3840*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^4*rq_per_P^5) + 10240*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 5120*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 10240*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 10240*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2880*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 13440*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 2880*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 1920*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 1920*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 35840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 15360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^5*rq_per_P^5) + 2048*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 10240*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 10240*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 8960*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 8192*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^6*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5)
    end

    if dorder_view==[4, 1, 1, 0]
        func = -16*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 + 96*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 96*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^2/rq_per_P^4 + 96*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 96*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 96*λ^2*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^4) - 704*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 96*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^4) - 704*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 1248*λ^2*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 1856*λ^2*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 1152*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 1152*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 2400*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 3008*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) + 1792*λ^3*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 768*λ^3*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) + 1792*λ^3*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 768*λ^3*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) + 1024*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 96*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^4) + 704*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 352*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 4608*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 1536*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^4) + 512*λ^4*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) + 512*λ^4*(-1 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) + 512*λ^4*(-1 - α)*(-2 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 1152*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 1152*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 96*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 704*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 1536*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^4) - 1024*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 768*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 768*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 512*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4)
    end

    if dorder_view==[3, 1, 1, 0]
        func = 16*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 + 96*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) - 96*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) + 16*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 + 96*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 96*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) + 288*λ^2*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 96*λ^2*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) + 192*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 480*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) - 96*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^3) + 128*λ^3*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) + 128*λ^3*(-1 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) - 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 96*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) + 32*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) + 256*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^3) - 192*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 96*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 96*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) - 64*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 128*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3)
    end

    if dorder_view==[2, 1, 1, 0]
        func = 4*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 + 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*cos(δ*π/rq_per_P)^2/rq_per_P^2 + 32*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) + 32*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) + 32*λ^2*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) + 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^2) + 32*λ^2*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^2) - 32*λ*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) - 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) - 16*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) - 32*λ^2*(-1 - α)*(-2 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2)
    end

    if dorder_view==[1, 1, 1, 0]
        func = -4*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P - 4*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(-2*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 2*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P - 8*λ*(-1 - α)*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P)
    end

    if dorder_view==[0, 1, 1, 0]
        func = -2*(2*λ*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2 - 4*λ*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/α^2
    end

    if dorder_view==[6, 0, 1, 0]
        func = 64*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 2048*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 1920*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 4800*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 2880*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 12160*λ*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^8/(α^3*rq_per_P^6) + 68160*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 78272*λ^2*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 48000*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 218240*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 90240*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 99840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 163840*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 23040*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 55296*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 8192*λ^6*(-6 - α)*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-7 - α)*sin(δ*π/rq_per_P)^8*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6)
    end

    if dorder_view==[5, 0, 1, 0]
        func = -64*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 + 1920*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 2048*λ*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 2880*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 15360*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 4800*λ^2*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 12160*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 19200*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1920*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 10240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 5120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 2048*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^7*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5)
    end

    if dorder_view==[4, 0, 1, 0]
        func = -16*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 128*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 96*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 96*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^4) + 736*λ*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 1248*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 1856*λ^2*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 1792*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 768*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 512*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4)
    end

    if dorder_view==[3, 0, 1, 0]
        func = 16*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 - 96*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 128*λ*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 288*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 96*λ^2*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) - 128*λ^3*(-1 - α)*(-2 - α)*(-3 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3)
    end

    if dorder_view==[2, 0, 1, 0]
        func = 4*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 + 8*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^2) - 40*λ*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) - 32*λ^2*(-1 - α)*(-2 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2)
    end

    if dorder_view==[1, 0, 1, 0]
        func = -4*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P - 8*λ*(-1 - α)*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P)
    end

    if dorder_view==[0, 0, 1, 0]
        func = -2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2
    end

    if dorder_view==[6, 2, 0, 0]
        func = 1920*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 1920*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 1920*λ^3*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 1920*λ^3*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 1920*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 1920*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 3840*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 3840*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 3840*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 3840*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 12032*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 44160*λ^3*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 44160*λ^3*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 1920*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 1920*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 5760*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 5760*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 12032*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 88320*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 88320*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 88320*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 88320*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 23040*λ^4*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 87040*λ^4*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 23040*λ^4*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 23040*λ^4*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 87040*λ^4*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 23040*λ^4*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 23040*λ^4*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 87040*λ^4*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 23040*λ^4*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 1920*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 1920*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 1920*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 1920*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 1920*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 1920*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 12032*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 132480*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 132480*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 69120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 261120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 69120*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 69120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 261120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 69120*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 69120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 261120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 69120*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) - 30720*λ^5*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 30720*λ^5*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 30720*λ^5*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 3840*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 3840*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 12032*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 44160*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 44160*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 44160*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 44160*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 6016*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 6016*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 138240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 522240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 138240*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^5*rq_per_P^6) - 122880*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 122880*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 122880*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 122880*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 122880*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 122880*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 122880*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 122880*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) - 8192*λ^6*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-2 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 8192*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 88320*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 88320*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 23040*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 87040*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 23040*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 87040*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 87040*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 23040*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 22080*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 307200*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 307200*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^6*rq_per_P^6) - 40960*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 40960*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 40960*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 40960*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 40960*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 261120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 69120*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 30720*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 30720*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 30720*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 122880*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^7*rq_per_P^6) + 122880*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 122880*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 8192*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 8192*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 8192*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 8192*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 8192*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 40960*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-6 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6)
    end

    if dorder_view==[5, 2, 0, 0]
        func = 1920*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 1920*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 1920*λ^3*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 8960*λ^3*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 1920*λ^3*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 64*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 + 1920*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1920*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 3840*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 17920*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 3840*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 3840*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 17920*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 3840*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) - 5120*λ^4*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 5120*λ^4*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 5120*λ^4*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 5120*λ^4*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 5120*λ^4*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 5120*λ^4*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1920*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 1920*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 5760*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 26880*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 5760*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^4*rq_per_P^5) - 15360*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 15360*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 15360*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 15360*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 15360*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 15360*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) - 2048*λ^5*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-1 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2048*λ^5*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 1920*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 1920*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 1920*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 8960*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 1920*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 1920*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 8960*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 1920*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 30720*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 30720*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^5*rq_per_P^5) - 8192*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 8192*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 8192*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 8192*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) + 3840*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 17920*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 3840*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) + 5120*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 5120*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 5120*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 5120*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 5120*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 20480*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^6*rq_per_P^5) + 15360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 15360*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 2048*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 2048*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 8192*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-5 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5)
    end

    if dorder_view==[4, 2, 0, 0]
        func = -96*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 96*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 96*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 96*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 704*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 768*λ^3*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 768*λ^3*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 704*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 1536*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 1536*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 1536*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 1536*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) - 512*λ^4*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 512*λ^4*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 512*λ^4*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 704*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 2304*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) + 2304*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^4) - 1536*λ^4*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 1536*λ^4*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 1536*λ^4*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 704*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 768*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 768*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 768*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 768*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 352*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 352*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 3072*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^4) + 1536*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 1536*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) + 512*λ^4*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 512*λ^4*(-1 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 512*λ^4*(-1 - α)*(-2 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 384*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 1536*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-4 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4)
    end

    if dorder_view==[3, 2, 0, 0]
        func = -96*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 96*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) - 128*λ^3*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 16*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 - 96*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) + 96*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^3*rq_per_P^3) - 256*λ^3*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) - 256*λ^3*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) + 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) - 96*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 384*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^3) + 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 96*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) + 128*λ^3*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 128*λ^3*(-1 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) + 256*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) - 64*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-3 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 64*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3)
    end

    if dorder_view==[2, 2, 0, 0]
        func = 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*cos(δ*π/rq_per_P)^2/rq_per_P^2 + 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 - 32*λ^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) - 32*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^2) + 32*λ^2*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) + 32*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) - 16*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-2 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) - 16*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2)
    end

    if dorder_view==[1, 2, 0, 0]
        func = -4*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*(4*λ*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - 4*λ^2*(-1 - α)*sin(δ*π/rq_per_P)^4/(α^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^2) + 4*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)))*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P
    end

    if dorder_view==[0, 2, 0, 0]
        func = (2*λ*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-α) + 4*λ^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/α^3
    end

    if dorder_view==[6, 1, 0, 0]
        func = -960*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 960*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 - 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 960*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 960*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 960*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) + 6016*λ^2*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) - 1920*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 1920*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 6016*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 22080*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 22080*λ^3*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) + 22080*λ^3*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) - 44160*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 44160*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 11520*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 43520*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 11520*λ^4*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 11520*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 43520*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 11520*λ^4*(-1 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 11520*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) - 43520*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) + 11520*λ^4*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 6016*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 34560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 130560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 34560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^4*rq_per_P^6) + 15360*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 15360*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) + 22080*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) + 61440*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 61440*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^5*rq_per_P^6) + 4096*λ^6*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 4096*λ^6*(-1 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 4096*λ^6*(-1 - α)*(-2 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) + 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) + 20480*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^6*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*(-2*λ*(-6 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6)
    end

    if dorder_view==[5, 1, 0, 0]
        func = -960*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) + 960*λ^2*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 64*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 - 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 960*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 4480*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 960*λ^3*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 960*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) - 4480*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) + 960*λ^3*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 1920*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 8960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 1920*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^3*rq_per_P^5) + 2560*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 2560*λ^4*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) + 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) + 7680*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 7680*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^4*rq_per_P^5) + 1024*λ^5*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 1024*λ^5*(-1 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 1024*λ^5*(-1 - α)*(-2 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) + 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) + 4096*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^5*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-2*λ*(-5 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5)
    end

    if dorder_view==[4, 1, 0, 0]
        func = 48*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 48*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 + 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 352*λ^2*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 352*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 384*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 384*λ^3*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) - 384*λ^3*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 768*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) - 768*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^4) + 256*λ^4*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 256*λ^4*(-1 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 256*λ^4*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4) + 352*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) + 768*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^4) - 384*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*(-2*λ*(-4 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4)
    end

    if dorder_view==[3, 1, 0, 0]
        func = 48*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) - 48*λ^2*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) + 16*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 + 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) - 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α^2*rq_per_P^3) + 64*λ^3*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 64*λ^3*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3) + 128*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^3) - 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 64*λ^3*(-1 - α)*(-2 - α)*(-2*λ*(-3 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3)
    end

    if dorder_view==[2, 1, 0, 0]
        func = 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 + 16*λ^2*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2) + 16*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^2) - 16*λ^2*(-1 - α)*(-2*λ*(-2 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2)
    end

    if dorder_view==[1, 1, 0, 0]
        func = -4*λ*(-2*λ*(-1 - α)*sin(δ*π/rq_per_P)^2/(α^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P
    end

    if dorder_view==[0, 1, 0, 0]
        func = (2*λ*sin(δ*π/rq_per_P)^2/(α*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)) - log(1 + 2*λ*sin(δ*π/rq_per_P)^2/α))*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-α)
    end

    if dorder_view==[6, 0, 0, 0]
        func = 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^6 - 64*λ*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^6 + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^6) + 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 960*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*cos(δ*π/rq_per_P)^6/(α^2*rq_per_P^6) - 6016*λ^2*(-1 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^6) + 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^6) - 22080*λ^3*(-1 - α)*(-2 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^6/(α^3*rq_per_P^6) + 43520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^6) - 11520*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^2/(α^3*rq_per_P^6) - 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^6/(α^4*rq_per_P^6) + 15360*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^4/(α^4*rq_per_P^6) - 4096*λ^6*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*(-5 - α)*π^6*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-6 - α)*sin(δ*π/rq_per_P)^6*cos(δ*π/rq_per_P)^6/(α^5*rq_per_P^6)
    end

    if dorder_view==[5, 0, 0, 0]
        func = -64*λ*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^5 + 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^5) - 960*λ^2*(-1 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^5/(α^2*rq_per_P^5) + 4480*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^5) - 960*λ^3*(-1 - α)*(-2 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)/(α^2*rq_per_P^5) - 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^5/(α^3*rq_per_P^5) + 2560*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^3/(α^3*rq_per_P^5) - 1024*λ^5*(-1 - α)*(-2 - α)*(-3 - α)*(-4 - α)*π^5*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-5 - α)*sin(δ*π/rq_per_P)^5*cos(δ*π/rq_per_P)^5/(α^4*rq_per_P^5)
    end

    if dorder_view==[4, 0, 0, 0]
        func = -16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^4 + 16*λ*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^4 - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^4/(α*rq_per_P^4) - 48*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*cos(δ*π/rq_per_P)^4/(α*rq_per_P^4) + 352*λ^2*(-1 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^4) - 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^4/(α^2*rq_per_P^4) + 384*λ^3*(-1 - α)*(-2 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^2/(α^2*rq_per_P^4) - 256*λ^4*(-1 - α)*(-2 - α)*(-3 - α)*π^4*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-4 - α)*sin(δ*π/rq_per_P)^4*cos(δ*π/rq_per_P)^4/(α^3*rq_per_P^4)
    end

    if dorder_view==[3, 0, 0, 0]
        func = 16*λ*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P^3 - 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)^3/(α*rq_per_P^3) + 48*λ^2*(-1 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)/(α*rq_per_P^3) - 64*λ^3*(-1 - α)*(-2 - α)*π^3*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-3 - α)*sin(δ*π/rq_per_P)^3*cos(δ*π/rq_per_P)^3/(α^2*rq_per_P^3)
    end

    if dorder_view==[2, 0, 0, 0]
        func = 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)^2/rq_per_P^2 - 4*λ*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*cos(δ*π/rq_per_P)^2/rq_per_P^2 - 16*λ^2*(-1 - α)*π^2*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-2 - α)*sin(δ*π/rq_per_P)^2*cos(δ*π/rq_per_P)^2/(α*rq_per_P^2)
    end

    if dorder_view==[1, 0, 0, 0]
        func = -4*λ*π*(1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-1 - α)*sin(δ*π/rq_per_P)*cos(δ*π/rq_per_P)/rq_per_P
    end

    if dorder_view==[0, 0, 0, 0]
        func = (1 + 2*λ*sin(δ*π/rq_per_P)^2/α)^(-α)
    end

    dorder[2] = dorder2  # resetting dorder[2]
    return powers_of_negative_one(dorder2) * float(func)  # correcting for amount of t2 derivatives

end


return rq_per_kernel, 3  # the function handle and the number of kernel hyperparameters
